
import contextlib
import sqlite3


@contextlib.contextmanager
def db_connection(DB_NAME: str = 'r.db') -> sqlite3.Connection:
    """上下文管理器用于数据库连接管理"""
    conn = sqlite3.connect(DB_NAME, detect_types=sqlite3.PARSE_COLNAMES)
    conn.row_factory = sqlite3.Row  # 允许以字典方式访问结果
    try:
        cursor = conn.cursor()
        yield cursor
        conn.commit()
    except Exception as e:
        conn.rollback()
        raise e
    finally:
        conn.close()



from xianbao.core.Task import TaskManager
from xianbao.core.DictData import DictManager
import xianbao.utils.FileScanner as FileScanner
from xianbao.core.BlockDeque import SQLiteProcessQueue

__all__ = ['TaskManager', 'DictManager', 'FileScanner', 'SQLiteProcessQueue']




from typing import Dict, List, Optional
from xianbao.core.SQLiteOp import db_connection


class DictManager:
    """数据字典管理类"""

    def __init__(self, db_name: str = "rpa_ztdb.db") -> None:
        """初始化数据字典管理器
        Args:
            db_name: SQLite数据库文件名，默认为 "rpa_ztdb.db"
        """
        self.DB_NAME = db_name
        self.initialize()

    def initialize(self) -> None:
        """初始化数据字典表结构"""
        with db_connection(self.DB_NAME) as cursor:
            # 创建字典类型表
            cursor.execute("""
            CREATE TABLE IF NOT EXISTS sys_dict_type (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                `name` TEXT NOT NULL,        -- 字典类型名称（如：性别字典）
                code TEXT NOT NULL UNIQUE, -- 字典类型编码（如：gender）
                description TEXT           -- 字典类型描述
            )
            """)

            # 创建字典项表
            cursor.execute("""
            CREATE TABLE IF NOT EXISTS sys_dict_data (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                `key` TEXT NOT NULL,         -- 字典项键（如：M）
                `value` TEXT NOT NULL,       -- 字典项值（如：男）
                type_id INTEGER NOT NULL,  -- 关联的字典类型ID
                sort_order INTEGER DEFAULT 0, -- 排序顺序
                FOREIGN KEY (type_id) REFERENCES sys_dict_type(id)
            )
            """)

            # 创建索引优化查询
            cursor.execute("CREATE UNIQUE INDEX IF NOT EXISTS idx_dict_type_code ON sys_dict_type(code)")
            cursor.execute("CREATE INDEX IF NOT EXISTS idx_dict_data_type_id ON sys_dict_data(type_id)")
            cursor.execute("CREATE INDEX IF NOT EXISTS idx_dict_data_key ON sys_dict_data(key)")

    # 字典类型管理
    def add_dict_type(self, name: str, code: str, description: str = "") -> int:
        """添加字典类型
        Args:
            name: 字典类型名称
            code: 字典类型编码（唯一标识）
            description: 类型描述（可选）
            
        Returns:
            新字典类型的ID
        """
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                "INSERT INTO sys_dict_type (name, code, description) "
                "VALUES (?, ?, ?)",
                (name, code, description)
            )
            return cursor.lastrowid

    def get_dict_type_by_code(self, code: str) -> Optional[Dict]:
        """根据编码获取字典类型
        Args:
            code: 字典类型编码
            
        Returns:
            字典类型信息（字典形式）或 None
        """
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                "SELECT id, name, code, description FROM sys_dict_type "
                "WHERE code = ?",
                (code,)
            )
            row = cursor.fetchone()
            return dict(row) if row else None

    def list_all_dict_types(self) -> List[Dict]:
        """获取所有字典类型
        Returns:
            字典类型列表
        """
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                "SELECT id, name, code, description FROM sys_dict_type "
                "ORDER BY id"
            )
            return [dict(row) for row in cursor.fetchall()]

    # 字典项管理
    def add_dict_item(
            self,
            type_code: str,
            key: str,
            value: str,
            sort_order: int = 0
    ) -> int:
        """添加字典项到指定类型的字典
        Args:
            type_code: 字典类型编码
            key: 字典项键
            value: 字典项值
            sort_order: 排序序号（可选）
            
        Returns:
            新字典项的ID
        """
        with db_connection(self.DB_NAME) as cursor:
            # 获取字典类型ID
            cursor.execute(
                "SELECT id FROM sys_dict_type WHERE code = ?",
                (type_code,)
            )
            type_row = cursor.fetchone()

            if not type_row:
                raise ValueError(f"字典类型不存在: {type_code}")

            type_id = type_row[0]

            cursor.execute(
                "INSERT INTO sys_dict_data (key, value, type_id, sort_order) "
                "VALUES (?, ?, ?, ?)",
                (key, value, type_id, sort_order)
            )
            return cursor.lastrowid

    def get_dict_items(self, type_code: str) -> List[Dict]:
        """根据字典类型编码获取所有字典项
        Args:
            type_code: 字典类型编码
            
        Returns:
            按sort_order排序的字典项列表
        """
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute("""
            SELECT d.key, d.value, d.sort_order 
            FROM sys_dict_data d
            JOIN sys_dict_type t ON d.type_id = t.id
            WHERE t.code = ?
            ORDER BY d.sort_order, d.id
            """, (type_code,))

            return [
                {"key": row[0], "value": row[1], "order": row[2]}
                for row in cursor.fetchall()
            ]

    def get_dict_value(self, type_code: str, key: str) -> Optional[str]:
        """获取指定字典类型和键对应的值
        Args:
            type_code: 字典类型编码
            key: 字典项键
            
        Returns:
            字典项值，如果不存在则返回None
        """
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute("""
            SELECT d.value 
            FROM sys_dict_data d
            JOIN sys_dict_type t ON d.type_id = t.id
            WHERE t.code = ? AND d.key = ?
            """, (type_code, key))

            row = cursor.fetchone()
            return row[0] if row else None

    def update_dict_item(
            self,
            type_code: str,
            key: str,
            new_value: str,
            new_sort_order: Optional[int] = None
    ) -> bool:
        """更新字典项
        Args:
            type_code: 字典类型编码
            key: 字典项键
            new_value: 新的字典项值
            new_sort_order: 新的排序序号（可选）
            
        Returns:
            是否成功更新
        """
        set_clause = "value = ?"
        params = [new_value]

        if new_sort_order is not None:
            set_clause += ", sort_order = ?"
            params.append(new_sort_order)

        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(f"""
            UPDATE sys_dict_data 
            SET {set_clause}
            WHERE id IN (
                SELECT d.id
                FROM sys_dict_data d
                JOIN sys_dict_type t ON d.type_id = t.id
                WHERE t.code = ? AND d.key = ?
            )
            """, (*params, type_code, key))

            return cursor.rowcount > 0

    # 高级功能
    def get_dict_as_mapping(self, type_code: str) -> Dict[str, str]:
        """将字典类型的所有项转换为字典映射
        Args:
            type_code: 字典类型编码

        Returns:
            {键: 值} 的字典
        """
        items = self.get_dict_items(type_code)
        return {item["key"]: item["value"] for item in items}

    def get_dict_lookup(self, type_code: str) -> Dict[str, str]:
        """获取反向字典（值到键的映射）
        Args:
            type_code: 字典类型编码
            
        Returns:
            {值: 键} 的字典
        """
        items = self.get_dict_items(type_code)
        return {item["value"]: item["key"] for item in items}


if __name__ == '__main__':
    # 创建字典管理器实例
    dict_manager = DictManager()


    # 创建新的字典类型
    user_type_id = dict_manager.add_dict_type(
        name="用户类型",
        code="user_type",
        description="用户类型分类"
    )

    # 添加字典项
    dict_manager.add_dict_item("user_type", "ADMIN", "管理员", 1)
    dict_manager.add_dict_item("user_type", "USER", "普通用户", 2)

    # 获取字典类型信息
    user_type = dict_manager.get_dict_type_by_code("user_type")
    if user_type:
        print(f"字典类型: {user_type['name']}({user_type['code']})")
    else:
        print("未找到用户类型字典")

    # 查询字典项列表
    status_items = dict_manager.get_dict_items("status")
    for item in status_items:
        print(f"{item['key']} => {item['value']}")

    # 获取单个值
    file_desc = dict_manager.get_dict_value("file_type", "pdf")
    print(f"PDF文件的描述: {file_desc}")  # 输出: PDF文档

    # 高级映射
    status_mapping = dict_manager.get_dict_as_mapping("status")
    print(status_mapping["INIT"])  # 输出: 初始状态

    # 反向查询
    status_lookup = dict_manager.get_dict_lookup("status")
    print(status_lookup["成功"])  # 输出: SUCC


from setuptools import setup, find_packages

setup(
    name='xianb',
    version='1.1.4',
    author='jxjiang',
    author_email='723137901@qq.com',
    description='',
    python_requires='>=3.8',
    install_requires=[],
    packages=find_packages()
)




import os
from collections import deque


def catalog_deep_list(folder: str, layer: int = 0) -> []:
    """扫描根目录下的日期格式目录
     Args:
        :param folder: 扫描的根目录路径
        :param layer: 上往下扫描多少层

     Returns:
         日期格式的目录相对路径列表，格式: ['20250701', '20250815', ...]

     Raises:
         ValueError: 如果根目录不存在
     """
    if not os.path.exists(folder):
        raise ValueError(f"目录不存在: {folder}")
    if not os.path.isdir(folder):
        raise ValueError(f"路径不是目录: {folder}")

    directories = []
    sta = deque([folder])
    layer_sta = deque([0])

    while len(sta) > 0:
        directory = sta.pop()
        current_layer = layer_sta.pop()

        if current_layer == layer or os.path.isfile(directory):
            directories.append(directory)
        elif os.path.isdir(directory):
            directory_list = os.listdir(directory)
            [sta.append(str(os.path.join(directory, c))) for c in directory_list]
            [layer_sta.append(current_layer + 1) for _ in range(1, len(directory_list) + 1)]

    return directories





import json
from datetime import datetime
from typing import Dict, List, Optional, Tuple
from xianbao.core.SQLiteOp import db_connection


class TaskManager:
    """RPA任务数据库管理器"""

    def __init__(self, db_name: str = "rpa_ztdb.db", table_name: str = "tasks") -> None:
        """初始化任务管理器
        Args:
            db_name: 数据库文件名，默认为 "rpa_ztdb.db"
            table_name: 表名，默认为 "tasks"
        """
        self.DB_NAME = db_name
        self.TABLE_NAME = table_name
        self.TIME_FORMAT = "%Y-%m-%d %H:%M:%S"

    def initialize_db(self) -> None:
        """初始化数据库和表结构"""
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(f"""
            CREATE TABLE IF NOT EXISTS {self.TABLE_NAME} (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                status INTEGER NOT NULL DEFAULT 0,  -- 使用整数代替文本
                meta TEXT NOT NULL,
                created_time TEXT NOT NULL,
                updated_time TEXT NOT NULL,
                business_type TEXT
            )
            """)
            # 索引
            cursor.execute(f"CREATE INDEX IF NOT EXISTS idx_status ON {self.TABLE_NAME}(status)")
            cursor.execute(f"CREATE INDEX IF NOT EXISTS idx_business_type ON {self.TABLE_NAME}(business_type)")
            cursor.execute(f"CREATE INDEX IF NOT EXISTS idx_created_time ON {self.TABLE_NAME}(created_time)")

    def add_task(self, meta: Dict, business_type: str, status: int = 0) -> int:
        """添加新任务（使用完整时间格式）"""
        now = datetime.now().strftime(self.TIME_FORMAT)  # 使用完整时间格式

        meta_json = json.dumps(meta)

        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"INSERT INTO {self.TABLE_NAME} (status, meta, created_time, updated_time, business_type) "
                "VALUES (?, ?, ?, ?, ?)",
                (status, meta_json, now, now, business_type)
            )
            return cursor.lastrowid

    def update_status(self, task_id: int, new_status: int) -> bool:
        """原子性更新任务状态
        Args:
            task_id: 任务ID
            new_status: 新状态

        Returns:
            更新是否成功
        """
        now = datetime.now().strftime(self.TIME_FORMAT)

        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"UPDATE {self.TABLE_NAME} SET status = ?, updated_time = ? "
                "WHERE id = ?",
                (new_status, now, task_id)
            )
            return cursor.rowcount > 0

    def update_status_and_meta(self, task_id: int, new_status: int, new_meta: Dict) -> bool:
        """
        同时更新任务状态和 meta 字段

        Args:
            task_id: 任务ID
            new_status: 新状态值
            new_meta: 更新后的 meta 数据（字典格式）

        Returns:
            是否成功更新任务
        """
        now = datetime.now().strftime(self.TIME_FORMAT)
        meta_json = json.dumps(new_meta)

        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"UPDATE {self.TABLE_NAME} SET status = ?, meta = ?, updated_time = ? WHERE id = ?",
                (new_status, meta_json, now, task_id)
            )
            return cursor.rowcount > 0

    def list_tasks_by_status(
            self,
            status: int,
            page: int = 1,
            per_page: int = 10,
            order_by: str = "id",
            ascending: bool = True
    ) -> List[Dict]:
        """根据状态分页查询任务
        Args:
            status: 状态值
            page: 页码(1-based)
            per_page: 每页数量
            order_by: 排序字段
            ascending: 是否升序

        Returns:
            任务字典列表(包含反序列化的meta)
        """
        offset = (page - 1) * per_page
        order_direction = "ASC" if ascending else "DESC"

        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"SELECT id, status, meta, created_time, updated_time, business_type "
                f"FROM {self.TABLE_NAME} "
                f"WHERE status = ? "
                f"ORDER BY {order_by} {order_direction} "
                "LIMIT ? OFFSET ?",
                (status, per_page, offset)
            )

            tasks = []
            for row in cursor.fetchall():
                task = dict(row)
                task["meta"] = json.loads(task["meta"])
                task["business_type"] = task.get("business_type")
                tasks.append(task)
            return tasks

    def count_by_status(self, status: int) -> int:
        """获取指定状态的记录数量
        Args:
            status: 状态值

        Returns:
            记录数量
        """
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"SELECT COUNT(*) FROM {self.TABLE_NAME} WHERE status = ?",
                (status,)
            )
            return cursor.fetchone()[0]

    def get_task(self, task_id: int) -> Optional[Dict]:
        """根据ID获取单个任务详情"""
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"SELECT * FROM {self.TABLE_NAME} WHERE id = ?",
                (task_id,)
            )
            row = cursor.fetchone()
            if not row:
                return None

            task = dict(row)
            task["meta"] = json.loads(task["meta"])
            task["business_type"] = task.get("business_type")
            return task

    def batch_update_status(self, task_ids: List[int], new_status: int) -> int:
        """批量更新任务状态
        Args:
            task_ids: 任务ID列表
            new_status: 新状态

        Returns:
            更新的记录数
        """
        if not task_ids:
            return 0

        now = datetime.now().strftime(self.TIME_FORMAT)  # 使用完整时间格式
        id_placeholders = ','.join(['?'] * len(task_ids))

        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"UPDATE {self.TABLE_NAME} SET status = ?, updated_time = ? "
                f"WHERE id IN ({id_placeholders})",
                (new_status, now, *task_ids)
            )
            return cursor.rowcount

    def delete_task(self, task_id: int) -> bool:
        """删除指定任务
        Args:
            task_id: 任务ID

        Returns:
            是否成功删除
        """
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"DELETE FROM {self.TABLE_NAME} WHERE id = ?",
                (task_id,)
            )
            return cursor.rowcount > 0

    def health_check(self) -> Tuple[bool, str]:
        """数据库健康检查
        Returns:
            (健康状态, 状态信息)
        """
        try:
            with db_connection(self.DB_NAME) as cursor:
                cursor.execute(f"SELECT 1 FROM {self.TABLE_NAME} LIMIT 1")
            return True, "Database is healthy"
        except Exception as e:
            return False, f"Database error: {str(e)}"

    def list_all_tasks(
            self,
            page: int = 1,
            per_page: int = 10,
            order_by: str = "id",
            ascending: bool = True
    ) -> List[Dict]:
        """分页查询所有任务（无状态过滤）

        Args:
            page: 页码(1-based)
            per_page: 每页数量
            order_by: 排序字段
            ascending: 是否升序

        Returns:
            任务字典列表(包含反序列化的meta)
        """
        offset = (page - 1) * per_page
        order_direction = "ASC" if ascending else "DESC"

        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"SELECT id, status, meta, created_time, updated_time, business_type "
                f"FROM {self.TABLE_NAME} "
                f"ORDER BY {order_by} {order_direction} "
                "LIMIT ? OFFSET ?",
                (per_page, offset)
            )

            tasks_data = []
            for row in cursor.fetchall():
                task = dict(row)
                task["meta"] = json.loads(task["meta"])
                task["business_type"] = task.get("business_type")
                tasks_data.append(task)
            return tasks_data

    def count_all_tasks(self) -> int:
        """获取整个表格的总记录数

        Returns:
            表格中的总记录数量
        """
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(f"SELECT COUNT(*) FROM {self.TABLE_NAME}")
            return cursor.fetchone()[0]

    def advanced_search(
            self,
            status: Optional[int] = None,
            created_start: Optional[str] = None,
            created_end: Optional[str] = None,
            meta_contains: Optional[Dict] = None,
            page: int = 1,
            per_page: int = 10
    ) -> Tuple[List[Dict], int]:
        """高级分页搜索（支持多种条件）

        Args:
            status: 任务状态过滤
            created_start: 创建时间开始(yyyy-MM-dd)
            created_end: 创建时间结束(yyyy-MM-dd)
            meta_contains: meta中应包含的键值对
            page: 页码
            per_page: 每页数量

        Returns:
            (匹配任务列表, 匹配总数量)
        """
        offset = (page - 1) * per_page
        conditions = []
        params = []

        # 构建查询条件
        if status:
            conditions.append("status = ?")
            params.append(int(status))

        if created_start:
            conditions.append("created_time >= ?")
            params.append(created_start + " 00:00:00")  # 添加时间部分以涵盖全天

        if created_end:
            conditions.append("created_time <= ?")
            params.append(created_end + " 23:59:59")  # 添加时间部分以涵盖全天

        # 处理JSON字段查询
        if meta_contains:
            for key, value in meta_contains.items():
                # JSON路径查找: $.key = value
                conditions.append(f"json_extract(meta, '$.\"{key}\"') = ?")
                params.append(str(value))

        where_clause = ""
        if conditions:
            where_clause = "WHERE " + " AND ".join(conditions)

        with db_connection(self.DB_NAME) as cursor:
            # 查询总数
            count_query = f"SELECT COUNT(*) FROM {self.TABLE_NAME} {where_clause}"
            cursor.execute(count_query, params)
            total_count = cursor.fetchone()[0]

            # 查询分页数据
            data_query = (
                f"SELECT id, status, meta, created_time, updated_time, business_type "
                f"FROM {self.TABLE_NAME} {where_clause} "
                f"ORDER BY created_time DESC "
                f"LIMIT ? OFFSET ?"
            )
            cursor.execute(data_query, params + [per_page, offset])

            tasks_data = []
            for row in cursor.fetchall():
                task = dict(row)
                task["meta"] = json.loads(task["meta"])
                task["business_type"] = task.get("business_type")
                tasks_data.append(task)

            return tasks_data, total_count

    def parse_datetime(self, dt_str: str) -> datetime:
        """将数据库时间字符串解析为datetime对象
        Args:
            dt_str: 格式为yyyy-MM-dd HH:mm:ss的字符串

        Returns:
            datetime对象
        """
        return datetime.strptime(dt_str, self.TIME_FORMAT)

    def format_datetime(self, dt: datetime) -> str:
        """将datetime对象格式化为数据库字符串
        Args:
            dt: datetime对象

        Returns:
            格式为yyyy-MM-dd HH:mm:ss的字符串
        """
        return dt.strftime(self.TIME_FORMAT)

    def list_tasks_created_between(self, start: datetime, end: datetime) -> List[Dict]:
        """查询在指定时间范围内创建的任务
        Args:
            start: 开始时间（datetime对象）
            end: 结束时间（datetime对象）

        Returns:
            任务列表
        """
        start_str = self.format_datetime(start)
        end_str = self.format_datetime(end)

        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"SELECT * FROM {self.TABLE_NAME} "
                "WHERE created_time BETWEEN ? AND ? "
                "ORDER BY created_time",
                (start_str, end_str)
            )

            tasks_data = []
            for row in cursor.fetchall():
                task = dict(row)
                task["meta"] = json.loads(task["meta"])
                task["business_type"] = task.get("business_type")
                # 添加解析后的datetime对象到结果
                task["created_datetime"] = self.parse_datetime(task["created_time"])
                task["updated_datetime"] = self.parse_datetime(task["updated_time"])
                tasks_data.append(task)
            return tasks_data

    def list_tasks_by_business_type(
        self,
        business_type: str,
        page: int = 1,
        per_page: int = 10,
        order_by: str = "id",
        ascending: bool = True
    ) -> List[Dict]:
        """根据业务类型分页查询任务"""
        offset = (page - 1) * per_page
        order_direction = "ASC" if ascending else "DESC"

        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"SELECT id, status, meta, created_time, updated_time, business_type "
                f"FROM {self.TABLE_NAME} "
                f"WHERE business_type = ? "
                f"ORDER BY {order_by} {order_direction} "
                "LIMIT ? OFFSET ?",
                (business_type, per_page, offset)
            )

            tasks = []
            for row in cursor.fetchall():
                task = dict(row)
                task["meta"] = json.loads(task["meta"])
                tasks.append(task)
            return tasks

    def count_tasks_by_business_type(self, business_type: str) -> int:
        """获取指定业务类型的记录数量"""
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(
                f"SELECT COUNT(*) FROM {self.TABLE_NAME} WHERE business_type = ?",
                (business_type,)
            )
            return cursor.fetchone()[0]

    def execute_custom_sql(self, sql: str, params: tuple = ()) -> List[Dict]:
        """
        执行自定义SQL查询（适用于 SELECT 操作）

        Args:
            sql: 完整的SQL查询语句
            params: SQL参数化查询所需的参数元组

        Returns:
            查询结果列表，每条记录为字典格式
        """
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(sql, params)
            rows = cursor.fetchall()

            # 将结果转换为字典形式
            columns = [desc[0] for desc in cursor.description]
            return [dict(zip(columns, row)) for row in rows]

    def execute_custom_update(self, sql: str, params: tuple = ()) -> int:
        """
        执行自定义SQL写入操作（适用于 INSERT/UPDATE/DELETE）

        Args:
            sql: 完整的SQL语句
            params: 参数化值

        Returns:
            受影响的行数
        """
        with db_connection(self.DB_NAME) as cursor:
            cursor.execute(sql, params)
            return cursor.rowcount



import sqlite3
import os
import threading
import time
from contextlib import contextmanager
from queue import Empty, Full
import logging


class SQLiteProcessQueue:
    """
    多进程安全的SQLite阻塞队列

    特性：
    - 进程安全的数据库连接管理
    - 基于文件锁的进程间协调
    - 可选的队列大小限制
    - 高效的数据获取策略
    - 指数退避的等待机制
    - 数据库维护功能

    Note: 每个进程应创建自己的队列实例，指向同一个数据库文件
    """

    def __init__(self, db_path='rpa_ztdb.db', table_name='queue',
                 max_size=10000, timeout=30.0,
                 lock_path='rpa_queue',
                 journal_mode='WAL', busy_timeout=5000,
                 poll_interval=0.1, max_poll_interval=1.0):
        """
        初始化多进程队列

        :param db_path: SQLite数据库文件路径
        :param table_name: 队列表名
        :param max_size: 队列最大容量(可选)
        :param timeout: 默认操作超时时间(秒)
        :param journal_mode: SQLite日志模式(WAL/DELETE等)
        :param busy_timeout: 数据库忙等待超时(毫秒)
        :param poll_interval: 队列空时初始轮询间隔(秒)
        :param max_poll_interval: 最大轮询间隔(秒)
        :param lock_path: 锁文件
        """
        self.lock_path = lock_path
        self.db_path = os.path.abspath(db_path)
        self.table_name = table_name
        self.max_size = max_size
        self.timeout = timeout
        self.poll_interval = poll_interval
        self.max_poll_interval = max_poll_interval
        self.journal_mode = journal_mode
        self.busy_timeout = busy_timeout
        self.fd = None

        # 为每个进程创建独立的数据库连接
        self.conn = None

        # 初始化数据库结构
        self._init_db()

    @contextmanager
    def _db_connection(self):
        """进程安全的数据库连接上下文管理"""
        try:
            if self.conn is None:
                # 创建数据库目录(如果需要)
                os.makedirs(os.path.dirname(self.db_path), exist_ok=True)

                # 创建新连接
                self.conn = sqlite3.connect(
                    self.db_path,
                    timeout=self.busy_timeout / 1000,
                    isolation_level=None  # 使用自动提交模式
                )

                # 优化设置
                self.conn.execute(f'PRAGMA journal_mode={self.journal_mode}')
                self.conn.execute('PRAGMA synchronous=NORMAL')
            yield self.conn
        except sqlite3.OperationalError as e:
            if "database is locked" in str(e):
                raise BusyError("Database is locked") from e
            raise
        except sqlite3.Error as e:
            logging.error(f"Database error: {e}")
            raise
        finally:
            # 不关闭连接，保持打开状态供后续使用
            pass

    def _init_db(self):
        """初始化数据库结构"""
        with self._db_connection() as conn:
            conn.execute(f'''
                CREATE TABLE IF NOT EXISTS {self.table_name} (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    data BLOB NOT NULL,
                    timestamp DATETIME DEFAULT CURRENT_TIMESTAMP
                );
            ''')
            # 加速消费查询
            conn.execute(f'CREATE INDEX IF NOT EXISTS idx_order ON {self.table_name}(id ASC);')
            # 检查点清理
            conn.execute('PRAGMA wal_checkpoint(TRUNCATE);')

    def _file_lock(self, timeout=10.0):
        """基于文件的锁，用于进程间协调"""
        start_time = time.monotonic()
        while True:
            try:
                # 尝试创建锁文件
                self.fd = os.open(self.lock_path, os.O_CREAT | os.O_EXCL | os.O_RDWR)
                os.close(self.fd)
                return
            except FileExistsError:
                # 检查超时
                if time.monotonic() - start_time > timeout:
                    raise TimeoutError("File lock timeout")
                # 指数退避等待
                time.sleep(min(0.1 * (2 ** int((time.monotonic() - start_time) / 0.1)), 0.6))
            except Exception as e:
                logging.error(f"File lock error: {e}")
                raise

    def _file_unlock(self):
        """释放文件锁"""
        try:
            if os.path.exists(self.lock_path):
                os.unlink(self.lock_path)
                print("Lock file removed", threading.currentThread().name)
        except Exception as e:
            logging.warning(f"Failed to remove lock file: {e}")

    def put(self, item: bytes, block=True, timeout=None):
        """
        将项目放入队列

        :param item: 要放入队列的数据(bytes)
        :param block: 如果队列满是否阻塞
        :param timeout: 阻塞超时时间(秒)
        :raises Full: 队列满且不阻塞或超时时
        """
        if not isinstance(item, bytes):
            raise TypeError("Only bytes are supported")

        timeout = timeout or self.timeout
        start_time = time.monotonic()
        poll_interval = self.poll_interval

        while True:
            # 检查队列大小(使用文件锁确保准确)
            try:
                self._file_lock()
                current_size = self.qsize()

                # 检查队列是否已满
                if self.max_size is not None and current_size >= self.max_size:
                    if not block:
                        raise Full("Queue full")

                    # 检查超时
                    elapsed = time.monotonic() - start_time
                    if elapsed >= timeout:
                        raise Full("Queue full")

                    # 等待并重试
                    time.sleep(poll_interval)
                    poll_interval = min(poll_interval * 2, self.max_poll_interval)
                    continue

                # 有空间，执行插入
                with self._db_connection() as conn:
                    conn.execute(
                        f'INSERT INTO {self.table_name} (data) VALUES (?)',
                        (item,)
                    )
                return
            finally:
                self._file_unlock()

    def get(self, block=True, timeout=None) -> bytes:
        """
        从队列中获取并移除项目

        :param block: 如果队列空是否阻塞
        :param timeout: 阻塞超时时间(秒)
        :return: 从队列中获取的数据(bytes)
        :raises Empty: 队列空且不阻塞或超时时
        """
        timeout = timeout or self.timeout
        start_time = time.monotonic()
        poll_interval = self.poll_interval

        while True:
            try:
                # 获取文件锁
                self._file_lock()

                # 尝试获取数据
                with self._db_connection() as conn:
                    cursor = conn.cursor()
                    cursor.execute(
                        f'DELETE FROM {self.table_name} '
                        'WHERE id = (SELECT id FROM {0} ORDER BY id ASC LIMIT 1) '
                        'RETURNING data'.format(self.table_name)
                    )
                    result = cursor.fetchone()
                    if result:
                        return result[0]

                # 队列为空，检查阻塞设置
                if not block:
                    raise Empty("Queue empty")

                # 检查超时
                elapsed = time.monotonic() - start_time
                if elapsed >= timeout:
                    raise Empty("Queue empty")

                # 等待并重试
                time.sleep(poll_interval)
                poll_interval = min(poll_interval * 2, self.max_poll_interval)
            finally:
                self._file_unlock()

    def qsize(self) -> int:
        """返回队列中的项目数"""
        with self._db_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(f'SELECT COUNT(*) FROM {self.table_name}')
            return cursor.fetchone()[0]

    def empty(self) -> bool:
        """如果队列为空返回True，否则False"""
        return self.qsize() == 0

    def full(self) -> bool:
        """如果队列满返回True，否则False"""
        if self.max_size is None:
            return False
        return self.qsize() >= self.max_size

    def clear(self):
        """清空队列"""
        try:
            self._file_lock()
            with self._db_connection() as conn:
                conn.execute(f'DELETE FROM {self.table_name}')
        finally:
            self._file_unlock()

    def vacuum(self):
        """回收数据库空间"""
        try:
            self._file_lock()
            with self._db_connection() as conn:
                conn.execute('VACUUM')
        finally:
            self._file_unlock()

    def maintenance(self):
        """执行数据库维护任务"""
        try:
            self._file_lock()
            with self._db_connection() as conn:
                # WAL检查点
                conn.execute('PRAGMA wal_checkpoint(TRUNCATE);')

                # 重新构建索引
                conn.execute(f'REINDEX {self.table_name}')
        finally:
            self._file_unlock()

    def close(self):
        """关闭数据库连接"""
        if self.conn:
            try:
                self.conn.close()
            except sqlite3.Error:
                pass
            finally:
                self.conn = None


class BusyError(Exception):
    """数据库忙异常"""
    pass
